#!/usr/bin/env python3
"""
SSL Certificate Analyzer
Advanced SSL/TLS certificate analysis and validation tool.
Parameters:
    - target: Domain or certificate file to analyze
    - mode: Analysis mode (domain, file)
    - output: Output file for results
Requires: cryptography, pyOpenSSL, requests packages
"""
import argparse
import logging
from datetime import datetime
import json
import ssl
import socket
import OpenSSL
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.x509.oid import NameOID
import requests
import idna
from urllib.parse import urlparse
import concurrent.futures
import hashlib
import base64

class SSLCertificateAnalyzer:
    def __init__(self, target, mode='domain'):
        self.target = target
        self.mode = mode.lower()
        self.results = {}
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def get_certificate_from_domain(self, domain, port=443):
        """Get SSL certificate from domain"""
        try:
            context = ssl.create_default_context()
            with socket.create_connection((domain, port)) as sock:
                with context.wrap_socket(
                    sock, server_hostname=domain
                ) as ssock:
                    cert_der = ssock.getpeercert(binary_form=True)
                    return x509.load_der_x509_certificate(
                        cert_der,
                        default_backend()
                    )
        except Exception as e:
            self.logger.error(
                f"Error getting certificate from {domain}: {str(e)}"
            )
            return None

    def get_certificate_from_file(self, file_path):
        """Get SSL certificate from file"""
        try:
            with open(file_path, 'rb') as f:
                cert_data = f.read()
                try:
                    return x509.load_pem_x509_certificate(
                        cert_data,
                        default_backend()
                    )
                except:
                    return x509.load_der_x509_certificate(
                        cert_data,
                        default_backend()
                    )
        except Exception as e:
            self.logger.error(
                f"Error reading certificate from {file_path}: {str(e)}"
            )
            return None

    def analyze_certificate(self, cert):
        """Analyze SSL certificate details"""
        try:
            cert_info = {
                'subject': {},
                'issuer': {},
                'validity': {},
                'extensions': [],
                'signature': {},
                'public_key': {},
                'serial_number': str(cert.serial_number),
                'version': cert.version.name
            }

            # Subject information
            for attr in cert.subject:
                cert_info['subject'][attr.oid._name] = attr.value

            # Issuer information
            for attr in cert.issuer:
                cert_info['issuer'][attr.oid._name] = attr.value

            # Validity period
            cert_info['validity'] = {
                'not_before': cert.not_valid_before.isoformat(),
                'not_after': cert.not_valid_after.isoformat(),
                'days_remaining': (
                    cert.not_valid_after - datetime.now()
                ).days
            }

            # Extensions
            for extension in cert.extensions:
                ext_info = {
                    'name': extension.oid._name,
                    'critical': extension.critical
                }
                
                # Handle specific extensions
                if extension.oid._name == 'subjectAltName':
                    ext_info['value'] = [
                        str(name) for name in extension.value
                    ]
                elif extension.oid._name == 'keyUsage':
                    ext_info['value'] = [
                        usage for usage in extension.value._names
                    ]
                elif extension.oid._name == 'extendedKeyUsage':
                    ext_info['value'] = [
                        usage._name for usage in extension.value
                    ]
                else:
                    ext_info['value'] = str(extension.value)
                
                cert_info['extensions'].append(ext_info)

            # Signature information
            cert_info['signature'] = {
                'algorithm': cert.signature_algorithm_oid._name,
                'value': base64.b64encode(
                    cert.signature
                ).decode('utf-8')
            }

            # Public key information
            public_key = cert.public_key()
            cert_info['public_key'] = {
                'algorithm': public_key.__class__.__name__,
                'size': public_key.key_size
            }

            return cert_info
        except Exception as e:
            self.logger.error(f"Error analyzing certificate: {str(e)}")
            return None

    def check_certificate_chain(self, domain):
        """Check certificate chain validity"""
        try:
            chain_info = {
                'valid': False,
                'chain': [],
                'issues': []
            }

            context = ssl.create_default_context()
            with socket.create_connection((domain, 443)) as sock:
                with context.wrap_socket(
                    sock, server_hostname=domain
                ) as ssock:
                    cert_chain = ssock.getpeercert(True)
                    chain_info['valid'] = True
                    
                    # Get certificate chain
                    cert = OpenSSL.crypto.load_certificate(
                        OpenSSL.crypto.FILETYPE_ASN1,
                        cert_chain
                    )
                    while cert:
                        chain_info['chain'].append({
                            'subject': dict(cert.get_subject().get_components()),
                            'issuer': dict(cert.get_issuer().get_components()),
                            'serial': cert.get_serial_number()
                        })
                        # Try to get the next certificate in chain
                        try:
                            cert = OpenSSL.crypto.load_certificate(
                                OpenSSL.crypto.FILETYPE_ASN1,
                                cert.get_issuer().der()
                            )
                        except:
                            break

            return chain_info
        except ssl.SSLError as e:
            chain_info['valid'] = False
            chain_info['issues'].append(str(e))
            return chain_info
        except Exception as e:
            self.logger.error(f"Error checking certificate chain: {str(e)}")
            return None

    def check_ciphers(self, domain):
        """Check supported SSL/TLS ciphers"""
        try:
            cipher_info = {
                'supported_protocols': [],
                'supported_ciphers': [],
                'insecure_ciphers': []
            }

            # Check TLS versions
            protocols = [
                ssl.PROTOCOL_TLSv1,
                ssl.PROTOCOL_TLSv1_1,
                ssl.PROTOCOL_TLSv1_2,
                ssl.PROTOCOL_TLSv1_3
            ]
            
            for protocol in protocols:
                try:
                    context = ssl.SSLContext(protocol)
                    with socket.create_connection((domain, 443)) as sock:
                        with context.wrap_socket(
                            sock, server_hostname=domain
                        ) as ssock:
                            cipher_info['supported_protocols'].append(
                                ssock.version()
                            )
                except:
                    continue

            # Check ciphers
            context = ssl.create_default_context()
            with socket.create_connection((domain, 443)) as sock:
                with context.wrap_socket(
                    sock, server_hostname=domain
                ) as ssock:
                    cipher = ssock.cipher()
                    cipher_info['supported_ciphers'].append({
                        'name': cipher[0],
                        'version': cipher[1],
                        'bits': cipher[2]
                    })

                    # Check for weak ciphers
                    if (
                        'RC4' in cipher[0] or
                        'MD5' in cipher[0] or
                        '3DES' in cipher[0]
                    ):
                        cipher_info['insecure_ciphers'].append(cipher[0])

            return cipher_info
        except Exception as e:
            self.logger.error(f"Error checking ciphers: {str(e)}")
            return None

    def check_vulnerabilities(self, domain):
        """Check for known SSL/TLS vulnerabilities"""
        try:
            vuln_info = {
                'heartbleed': False,
                'poodle': False,
                'freak': False,
                'logjam': False,
                'beast': False,
                'lucky13': False,
                'issues': []
            }

            # Check for Heartbleed
            try:
                context = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
                context.verify_mode = ssl.CERT_NONE
                with socket.create_connection((domain, 443)) as sock:
                    with context.wrap_socket(sock) as ssock:
                        if hasattr(ssock, 'heartbeat_supported'):
                            vuln_info['heartbleed'] = True
            except:
                pass

            # Check for POODLE
            try:
                context = ssl.SSLContext(ssl.PROTOCOL_SSLv3)
                with socket.create_connection((domain, 443)) as sock:
                    with context.wrap_socket(sock) as ssock:
                        vuln_info['poodle'] = True
            except:
                pass

            # Check cipher strengths for other vulnerabilities
            cipher_info = self.check_ciphers(domain)
            if cipher_info:
                for cipher in cipher_info['supported_ciphers']:
                    if 'EXPORT' in cipher['name']:
                        vuln_info['freak'] = True
                    if 'DHE_EXPORT' in cipher['name']:
                        vuln_info['logjam'] = True
                    if cipher['version'] in ['TLSv1.0']:
                        vuln_info['beast'] = True
                    if 'CBC' in cipher['name']:
                        vuln_info['lucky13'] = True

            return vuln_info
        except Exception as e:
            self.logger.error(f"Error checking vulnerabilities: {str(e)}")
            return None

    def check_configuration(self, domain):
        """Check SSL/TLS configuration best practices"""
        try:
            config_info = {
                'hsts': False,
                'secure_renegotiation': False,
                'compression': False,
                'session_tickets': False,
                'ocsp_stapling': False,
                'issues': []
            }

            # Check HSTS
            try:
                response = requests.get(f"https://{domain}")
                if 'Strict-Transport-Security' in response.headers:
                    config_info['hsts'] = True
            except:
                pass

            # Check other configurations
            context = ssl.create_default_context()
            with socket.create_connection((domain, 443)) as sock:
                with context.wrap_socket(
                    sock, server_hostname=domain
                ) as ssock:
                    # Check secure renegotiation
                    if hasattr(ssock, 'secure_renegotiation_supported'):
                        config_info['secure_renegotiation'] = (
                            ssock.secure_renegotiation_supported()
                        )

                    # Check compression
                    if hasattr(ssock, 'compression'):
                        config_info['compression'] = bool(ssock.compression())

                    # Check session tickets
                    if hasattr(ssock, 'session_tickets_supported'):
                        config_info['session_tickets'] = (
                            ssock.session_tickets_supported()
                        )

                    # Check OCSP stapling
                    if hasattr(ssock, 'ocsp_response'):
                        config_info['ocsp_stapling'] = (
                            bool(ssock.ocsp_response)
                        )

            return config_info
        except Exception as e:
            self.logger.error(f"Error checking configuration: {str(e)}")
            return None

    def generate_report(self):
        """Generate analysis report"""
        try:
            report = {
                'timestamp': datetime.now().isoformat(),
                'target': self.target,
                'mode': self.mode,
                'results': self.results
            }

            # Add summary section
            summary = {
                'valid': self.results.get('chain_info', {}).get('valid', False),
                'expires_in_days': self.results.get(
                    'certificate_info', {}
                ).get('validity', {}).get('days_remaining'),
                'issues_found': len(
                    self.results.get('vulnerability_info', {}).get(
                        'issues', []
                    )
                )
            }

            report['summary'] = summary
            return report
        except Exception as e:
            self.logger.error(f"Error generating report: {str(e)}")
            return None

    def save_report(self, report, output_file):
        """Save analysis report to file"""
        try:
            with open(output_file, 'w') as f:
                json.dump(report, f, indent=4)
            self.logger.info(f"Report saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving report: {str(e)}")

    def print_summary(self, report):
        """Print analysis summary"""
        self.logger.info("\nSSL Certificate Analysis Summary:")
        self.logger.info(f"Target: {report['target']}")
        self.logger.info(f"Analysis Mode: {report['mode']}")
        
        if 'summary' in report:
            summary = report['summary']
            self.logger.info("\nCertificate Status:")
            self.logger.info(f"Valid: {summary['valid']}")
            self.logger.info(
                f"Expires in: {summary['expires_in_days']} days"
            )
            self.logger.info(f"Issues Found: {summary['issues_found']}")

        if 'certificate_info' in report['results']:
            cert_info = report['results']['certificate_info']
            self.logger.info("\nCertificate Information:")
            self.logger.info(
                f"Subject: {cert_info['subject'].get('commonName')}"
            )
            self.logger.info(
                f"Issuer: {cert_info['issuer'].get('commonName')}"
            )
            self.logger.info(
                f"Valid Until: {cert_info['validity']['not_after']}"
            )

        if 'vulnerability_info' in report['results']:
            vuln_info = report['results']['vulnerability_info']
            self.logger.info("\nVulnerability Assessment:")
            for vuln, present in vuln_info.items():
                if vuln != 'issues':
                    self.logger.info(f"{vuln.title()}: {'Yes' if present else 'No'}")

    def start_analysis(self, output_file=None):
        """Start SSL certificate analysis"""
        self.logger.info("Starting SSL certificate analysis")
        self.logger.info(f"Target: {self.target}")
        self.logger.info(f"Mode: {self.mode}")
        
        try:
            # Get certificate
            if self.mode == 'domain':
                cert = self.get_certificate_from_domain(self.target)
                if cert:
                    self.results['certificate_info'] = (
                        self.analyze_certificate(cert)
                    )
                    self.results['chain_info'] = (
                        self.check_certificate_chain(self.target)
                    )
                    self.results['cipher_info'] = (
                        self.check_ciphers(self.target)
                    )
                    self.results['vulnerability_info'] = (
                        self.check_vulnerabilities(self.target)
                    )
                    self.results['configuration_info'] = (
                        self.check_configuration(self.target)
                    )
            else:
                cert = self.get_certificate_from_file(self.target)
                if cert:
                    self.results['certificate_info'] = (
                        self.analyze_certificate(cert)
                    )
            
            # Generate and save report
            report = self.generate_report()
            if report and output_file:
                self.save_report(report, output_file)
            if report:
                self.print_summary(report)
                
        except KeyboardInterrupt:
            self.logger.info("Analysis interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during analysis: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="SSL Certificate Analyzer"
    )
    parser.add_argument(
        "target",
        help="Domain or certificate file to analyze"
    )
    parser.add_argument(
        "--mode",
        choices=['domain', 'file'],
        default='domain',
        help="Analysis mode"
    )
    parser.add_argument(
        "--output",
        help="Output file for analysis report"
    )
    args = parser.parse_args()

    analyzer = SSLCertificateAnalyzer(args.target, args.mode)
    analyzer.start_analysis(args.output)
